@@ -30,6 +30,9 @@
 extern "C" {
 #endif
 
+#include <stdlib.h>
+#include "uvm_dpi.h"
+#include "uvm_common.c"
 #include "uvm_regex.cc"
 #include "uvm_hdl.c"
 #include "uvm_svcmd_dpi.c"
